from netgen.geom2d import unit_square
from ngsolve import *
ngsglobals.msg_level = 2

mesh = Mesh (unit_square.GenerateMesh(maxh=0.3))

V = H1(mesh, order=3, dirichlet=[1,2,3,4])
u,v = V.TnT()
a = BilinearForm(V)
a += SymbolicBFI( grad(u) * grad(v) + 5*u*u*v- 1 * v)

def SimpleNewtonSolve(gfu,a,tol=1e-13,maxits=25):
    res = gfu.vec.CreateVector()
    du = gfu.vec.CreateVector()
    fes = gfu.space
    for it in range(maxits):
        print ("Iteration {:3}:".format(it))

        # compute resual A(u_i)
        a.Apply(gfu.vec, res)
        print("\t||res||_l2 = {:}".format(sqrt(InnerProduct(res,res))))
        
        # compute update and update solution
        a.AssembleLinearization(gfu.vec)
        du.data = a.mat.Inverse(fes.FreeDofs()) * res
        gfu.vec.data -= du

        #stopping criteria
        stopcritval = sqrt(abs(InnerProduct(du,res)))
        print("\t||update||_l2 = {:}".format(sqrt(InnerProduct(du,du))))
        print ("\t<A u",it,", A u",it,">_{-1}^0.5 = ", stopcritval)
        if stopcritval < tol:
            break

gfu = GridFunction(V)
Draw(gfu,mesh,"u")
SimpleNewtonSolve(gfu,a)